;
; chrout.asm - CHROUT kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Chrout (A=character out)
;
; Used registers: - (all preserved)
; Status: C (N Z for Accu)
; C=1:
; A=7 (not output file)
; Status response:
; ST=$00 if no error
; ST=$03 timeout
; ST=$80 if device not present
;Note:
;Currently buffer is 256 bytes long and only used for error channel...
;**************************************************
newchrout_t	.proc
		sta cfg_open
		.if SUPERCPU=1
		xba
		.else
		sta help_trap_accu_exp
		.fi

		lda buffer_output_exp
		eor #STDOUT
		bne nonstandard
		lda buffer_input_exp
		cmp #STDIN
		bne nonstandard
		#psr gotobank3back
		gcs standard

nonstandard	#psr switch_16k
standard	.if SUPERCPU=1
		xba
		.else
		lda help_trap_accu_exp
		.fi

newchrout	.proc
		pha
		#stx_l_a help_x_exp
		#ldatax buffer_output_exp
		lda buffer_info_exp,x
dback
		.if SUPERCPU=1
		lsr
		bcs chrout_not_data_sector
                
;BLK_WRITE_DIRTY, BLK_RW_DIRTY, BLK_RW_EOF_DIRTY

		lda $a6
		pha
		lda #1
		sta $a6			;SCPU Page 1

hard_chrout	lda buffer_address_low_exp,x
		inc a
		sta buffer_address_low_exp,x
		.else
		asr #$01
		bcs chrout_not_data_sector

hard_chrout	isb buffer_address_low_exp,x
		eor #$ff
		.fi

		beq chrout_nxt_bank
		sta $a4
		lda buffer_address_hi_exp,x
		bmi chrout_data_sector_last
chrout_bnk_nen2	sta $a5
chrout_bnk_nend

		.if SUPERCPU=1		;Store byte
		plx
		pla
		sta [$a4]
		xba
		stx $a6			;Restore $a6
		lda help_x_exp
		tax
		xba			;Set NZ flags
		.else
		pla
		ldx #0
		sta ($a4,x)
		ldx help_x_exp
		ora #$00		;Set NZ flags
		.fi
		jmp exit2basic
;---------------------------------
chrout_nxt_bank	sta $a4
		.if SUPERCPU=1
		lda buffer_address_hi_exp,x
		inc a
		sta buffer_address_hi_exp,x
		.else
		isb buffer_address_hi_exp,x
		eor #$ff
		.fi
		cmp buffer_address_end_hi_exp,x
		bcc chrout_bnk_nen2
		ora #$80
		sta buffer_address_hi_exp,x;Mark next bank

chrout_data_sector_last
		and #$7f
		sta $a5
		lda buffer_address_low_exp,x
		cmp buffer_address_end_low_exp,x
		bcc chrout_bnk_nend
		gcs chrout_next

newconsoljmp	jmp newconsole

;BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, BLK_READ, BLK_RW, STATUS, KERNAL, IGNORE

chrout_not_data_sector = *
		cpx #STDOUT
		beq newconsoljmp

		#stx_l_a buffer_currentp_exp
		lda buffer_drive_exp,x
		sta current_drive_exp
		lda buffer_info_exp,x
		cmp #INFO_KERNAL
		bne nchrout_stand

		lda current_drive_exp
		cmp #4
		bcc vew			;serial device?
		#ldatax help_x_exp
		jsr OP_cfg16up
		pla
                .if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr iecout
                .else
                jsr $eddd
                .fi
		pha
		jsr OP_cfg16down
		pla
		jmp exit2basic

vew		#psr $f1cd
		#ldatax help_x_exp
		lda current_drive_exp
		jmp exit2basic

;BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, BLK_READ, BLK_RW, STATUS, IGNORE

nchrout_stand	cmp #INFO_BLK_WRITE
		beq brt
		cmp #INFO_BLK_RW
		beq brt2
		cmp #INFO_BLK_RW_EOF
		bne nchrout_dodirty

		lda #INFO_BLK_RW_EOF_DIRTY
		.byte SKIP2
brt2		lda #INFO_BLK_RW_DIRTY
		.byte SKIP2
brt		lda #INFO_BLK_WRITE_DIRTY
		pha
		lda #INFO_IGNORE
		sta buffer_info_exp,x
                lda buffer_type_exp,x
                cmp #BUFFERMODE_FILE
                beq jo
                brk

jo		#phy_a
		jsr OP_save_zeropage
		jsr XIT_alloc
		jsr OP_restore_zeropage
		#ply_a

		#ldatax buffer_currentp_exp
		pla
		bcs nchrout_err
		sta buffer_info_exp,x
		gcc dback

XIT_alloc	#xit_init
		jsr XIT_save_tree	;-=> error
		clc
		rts

nchrout_err	lda #$83		;INFO_IGNORE
		sta Status
		#ldatax help_x_exp
		pla
		clc
		jmp exit2basic
;---------------------------------------

;BLK_R_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_READ, STATUS, IGNORE

nchrout_dodirty	cmp #INFO_STATUS
		bne nchrout_err

		lda buffer_start,x
		sta $a5
		lda buffer_address_low_exp,x
		sta $a4
		adc #1-1
		bcs nchrout_err
		sta buffer_address_low_exp,x
		.if SUPERCPU=1
		ldx $a6
		lda #1
		sta $a6
		pla
		sta [$a4]
		xba
		stx $a6
		lda help_x_exp
		tax
		xba
		.else
		ldx #0
		pla
		sta ($a4,x)
		ldx help_x_exp
		ora #0
		.fi
		clc
		jmp exit2basic
;---------------------------------------
chrout_next	#stx_l_a buffer_currentp_exp
		lda buffer_drive_exp,x
		sta current_drive_exp
		#phy_a
                #OP_scpu01up
		jsr OP_save_zeropage

		jsr XIT_do_next

		jsr OP_restore_zeropage
                #OP_scpu01down
		#ply_a
		bcs nchrout_err

		#ldatax buffer_currentp_exp
		.if SUPERCPU=0
		lda #0
		.fi
		gcc hard_chrout
;---------------------------------------
XIT_do_next	#xit_init

		#ldatax buffer_currentp_exp
		lda buffer_type_exp,x
		cmp #BUFFERMODE_DIRECT
		beq XIT_do_direct
		cmp #BUFFERMODE_FILE
		beq XIT_do_filenext
		cmp #BUFFERMODE_PCLINK
		beq XIT_do_pclink
		brk

XIT_do_pclink
XIT_do_direct	jsr OP_init_buffer_pointers
		clc
		rts
;---------------------------------------
;INFO_BLK_WRITE_DIRTY, INFO_BLK_RW_DIRTY, INFO_BLK_RW_EOF_DIRTY
;
XIT_do_filenext	#ldatax buffer_currentp_exp
		lda buffer_info_exp,x
		pha
		lda #INFO_IGNORE	;be safe
		sta buffer_info_exp,x

		lda buffer_address_end_hi_exp,x
		beq dc
		lda $a5			;buffer_address_hi_exp,x
		cmp buffer_address_max_hi_exp,x
		beq dc2			;eof mark?

		pla
		lda #INFO_BLK_RW_EOF_DIRTY
		pha
		jmp chrin_chrout_do_file_end.more

dc2		jsr XIT_flush_buf

		#ldatax buffer_currentp_exp
		lda buffer_address_max_hi_exp,x
		sec
		sbc buffer_start,x
		clc
		adc buffer_filepos2_exp,x
		sta buffer_filepos2_exp,x
		bcc dc
		#inc_l_x_a buffer_filepos3_exp
		bne dc
		#inc_l_x_a buffer_filepos4_exp;!!!!! (4Gb)

dc		jsr XIT_save_tree	;-=> error

		jmp chrin_chrout_do_file_end
		.pend

XIT_flush_buf	.proc
		#ldatax buffer_currentp_exp
		lda buffer_filepos2_exp,x
		jsr buffer_tree

		ldy #0
		#lda_zp_y pom1
		and #%01001111
		iny
		#ora_zp_y pom1
		iny
		#ora_zp_y pom1
		iny
		#ora_zp_y pom1
		bne allre		;is it already allocated?

		.comment
		lda buffer_address_low_exp,x
		clc
		adc #1
		tay
		#stz_a pom1
		lda buffer_address_hi_exp,x
		adc #0
		and #$7f
		sta pom2

vj2		lda #0			;fill with 0
vj		#sta_zp_y pom1
		iny
		bne vj
		inc pom2
		lda pom2
		lsr
		bcs vj2
		.endc

		jsr restore_allocator_state

		jsr find_free_sector	;allocate data sector

		#ldatax buffer_currentp_exp
		lda buffer_filepos2_exp,x
		jsr buffer_tree

		ldx #3
		ldy #3
vws		lda next_sector,x
		#sta_zp_y pom1
		dey
		dex
		bne vws
		#lda_zp_y pom1		;fill in
		and #%10110000
		ora next_sector
		#sta_zp_y pom1

		jsr save_allocator_state

allre		jmp XIT_write_out_buffer
		.pend
