;
; miscutil.asm - misc util
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

;--------------------------------------
; Select drive and set next sector
;
XIT_upload_next_sector .proc
		jsr OP_get_current_drive_type
		and #FS_MASK
		beq XIT_upload_next_sector2.upunknownfs

XIT_upload_next_sector2 .proc
		lda next_sector
		jsr head_select_drive
		lda next_sector+3
		sta low_reg
		sta sector_g
		lda next_sector+2
		sta low_reg
		sta cyllow_g
		lda next_sector+1
		sta low_reg
		sta cylhi_g
		#jmps LWOP_reporterror_74_xit
upunknownfs	sec
		gcs LWOP_reporterror_75_xit
		.pend
;-----------------------------------
; Read in a sector
;
XIT_read_a_sector2 .proc
		pha
		jsr XIT_upload_next_sector2
		jmp XIT_read_a_sector.in

XIT_read_a_sector .proc
		pha
		jsr XIT_upload_next_sector;-=> error
in		ldy #1
		jsr OP_read
		jsr LWOP_reporterror_read_xit;-=> error
		pla
		jsr OP_read512_here
		jsr OP_get_current_drive_type
		and #$c0
		eor #$80
		bne ji			;cdrom?
		jsr OP_read512
		jsr OP_read512
		jsr OP_read512
ji		jsr ata_pi_nomore_data
		jmp LWOP_reporterror_read_xit;-=> error
		.pend
;-----------------------------------
; Write out a sector
;
XIT_write_a_sector2 .proc
		pha
		jsr XIT_upload_next_sector2
		jmp XIT_write_a_sector.in

XIT_write_a_sector .proc
		pha
		jsr XIT_upload_next_sector;-=> error
in		ldy #1
		jsr OP_write
		pla
		bcs eer
		bne nonempty
		ldy #0
		sty low_reg
		sty hig_reg
ni		sta data_g
		sta data_g
		iny
		bpl ni
		gmi gd

nonempty	jsr init_pom1_y

		sec
		.byte SKIP1
lp		clc
lp2		#lda_zp_y pom1
		sta low_reg		;0-7 BIT
		iny

		#lda_zp_y pom1
		sta hig_reg		;8-15 BIT
		sta data_g
		iny
		bne lp2
		inc pom2
		bcs lp
gd		jsr ata_pi_nomore_data
		bcs eer
		jsr wait_busy		;wait until writing finished
eer		jmp LWOP_reporterror_write_xit;-=> error
		.pend
;-----------------------------------
; Loads a tree sector and create if non-existent
;
XIT_load_tree	.proc
		#lda0
		.byte SKIP2
;-----------------------------------
; Loads a tree sector (walks tree)
;
XIT_save_tree	.proc
m		= pom4
m2		= pom6
level		= pom8

		lda #1
		sta walk_tree_mode_exp
		lda #>b_cfs_file_misc_exp
		jsr buffer_subpart_cp

		ldy #<b_cfs_file_treepos_exp
		lda buffer_filepos3_exp,x;need to search for a new tree sector?
		sta m
		#cmp_zp_y current_path
		bne tr
		iny
		lda buffer_filepos4_exp,x
		#cmp_zp_y current_path
		bne tr2			;same as current tree sector!
ki		rts			;-=> ok

tr		#sta_zp_y current_path	;save current fp
		iny
tr2		lda buffer_filepos4_exp,x
		#sta_zp_y current_path
		sta m+1
		lda walk_tree_mode_exp
		beq bf			;flush tree sector

		jsr OP_load_next_sector_tree;restore tree sector
		lda buffer_start,x
		ora #>b_cfs_file_tree_exp;write out the tree sector
		jsr XIT_write_a_sector	;-=> error
		jsr restore_allocator_state

bf		ldx #3
		ldy #<(b_cfs_file_treeroot_exp+3)
idh		#lda_zp_y current_path
		sta next_sector,x
		dey
		dex
		bpl idh

		ldy #0
		sty m2
		sty m2+1

		lda m
idfy		tax
		ora m+1
		beq en

		txa
		bne *+4
		dec m+1
		dex
		txa

		lsr m+1
		ror
		rol m2
		rol m2+1
		lsr m+1
		ror
		rol m2
		rol m2+1
		lsr m+1
		ror
		rol m2
		rol m2+1

		iny
		gne idfy

en		sty level

loop		#ldatax buffer_currentp_exp
		lda buffer_start,x
		ora #>b_cfs_file_tree_exp
		jsr XIT_read_a_sector	;-=> error

loop2		dec level
		bmi ki
		lda walk_tree_mode_exp
		beq ccs

		#cnext next_sector , created_new_tree_sector_exp;remember tree sector position

		#lda0
ccs		tay
		lsr m2+1
		ror m2
		ror
		lsr m2+1
		ror m2
		ror
		lsr m2+1
		ror m2
		sta current_path
		#ldatax buffer_currentp_exp
		lda buffer_start,x
		adc #>b_cfs_file_tree_exp
		sta current_path+1
		ldx #3
mic		#lda_zp_y current_path
		asl
		asl
		asl
		rol next_sector,x
		asl
		rol next_sector,x
		tya
		clc
		adc #4
		tay
		and #15
		bne mic
		dex
		bpl mic
		lda next_sector
		ora next_sector+1
		ora next_sector+2
		ora next_sector+3
		bne loop
		lda walk_tree_mode_exp	;create nonexistent?
		beq oops

		jsr find_free_sector

		jsr OP_save_next_sector_treep

		#lda0
		jsr XIT_write_a_sector	;save empty tree sector

		ldy #0
		ldx #3
mi		#lda0
mi2		rol next_sector,x
		rol
		rol next_sector,x
		rol
		asl
		asl
		asl
		asl
		#ora_zp_y current_path
		#sta_zp_y current_path
		tya
		adc #4
		tay
		and #15
		bne mi
		dex
		bpl mi2

		jsr XIT_find_free_flush	;flush

		#cnext created_new_tree_sector_exp , next_sector;restore tree sector position

		lda current_path+1
		and #$fe
		jsr XIT_write_a_sector	;save modifyed tree sector
		jsr clr
		jsr OP_load_next_sector_treep
		jsr save_allocator_state
		jmp loop2

oops		lda #>b_cfs_file_misc_exp; Big hole in file so mark tree sector as unavailable
		jsr buffer_subpart_cp

		ldy #<b_cfs_file_treepos_exp
		lda #$ff
		#sta_zp_y current_path
		iny
		#sta_zp_y current_path

clr		lda current_path+1
		and #$fe
		sta current_path+1
		#lda0
		sta current_path
		tay
		jsr vd			;clear tree sector
		inc current_path+1
vd		#sta_zp_y current_path
		iny
		bne vd
		rts
		.pend
;--------------------------------------
;Init drive (eliminate possible diskchange problems...)
;--------------------------------------
XIT_init_drive2	.proc
		jsr OP_get_current_drive_type
		bpl noatapi
		lda #ATAPI_TEST_UNIT_READY
		jsr OP_atapi_packet
		jsr OP_atapi_command
		cmp #EATAPIERROR
		bne n
		lda error_g
		lda low_reg
		cmp #ATAPI_SENSE_UNIT_ATTENTION
		bne noatapi
		jsr invalidate_drive
n		jsr OP_get_current_drive_type
noatapi		rts
		.proc

XIT_init_drive	.proc
		jsr XIT_init_drive2
		and #FS_MASK
		clc
		beq OP_XIT_init_filesystem_from_disk
		rts
		.pend
;--------------------------------------
;Check drive is ready, and init it
;--------------------------------------
check_drive	.proc
		jsr head_select_drive
		bcc ok			;there's something bad
		lda #nop_d		;this helps freezed cdroms to
		sta low_reg		;come back to life...
		sta feat_g		;for atapi devices
		jmp ata_command		;now hope the best...
ok		rts
		.pend
;--------------------------------------
;Calculate linear access from tree sector
;--------------------------------------
linearaccess	.proc
		.if SUPERCPU=1
		lda pom3
		pha
		lda #1
		sta pom3
		.else
		lda #1
		.fi
		sta load_sectors_exp

		lda pom2
		pha
		lda pom1
		pha

		lda load_p_exp
		asl
		asl
		sta pom1
		tya
		adc #0
		sta pom2

		ldy #3
vcs		#lda_zp_y pom1
		sta next_sector,y
		dey
		bpl vcs

		lda load_size_exp	;how many sectors left?
		adc #<511		;511 for rounding
		lda load_size_exp+1
		adc #>511
		jsr apply_limit

linlp		#inc_l_a load_p_exp
		bmi gy

		#dec_l_a load_limit_exp
		beq gy

		ldy #3
		bit next_sector
		bvc chs

		clc
		#lda_zp_y pom1
		adc #1
		ldy #7
		#eor_zp_y pom1
		bne gy
		ldy #2
		#lda_zp_y pom1
		adc #0
		ldy #6
		#eor_zp_y pom1
		bne gy
		ldy #1
		#lda_zp_y pom1
		adc #0
		ldy #5
		#eor_zp_y pom1
		bne gy
		ldy #0
		#lda_zp_y pom1
		adc #0
		ldy #4
		#eor_zp_y pom1
		and #15
		beq vrg

gy		pla
		sta pom1
		pla
		sta pom2
		.if SUPERCPU=1
		pla
		sta pom3
		.fi
		rts

chs		#lda_zp_y pom1		;sector
		cmp drive_maxsector_exp,x
		bcc *+4
		lda #255
		adc #1
		ldy #7
		#eor_zp_y pom1
		bne gy
		ldy #0			;head
		#lda_zp_y pom1
		and #15
		bcc vk1
		cmp drive_maxhead_exp,x
		bcc *+4
		lda #254
		adc #1
vk1		ldy #4
		#eor_zp_y pom1
		and #15
		bne gy
		ldy #2			;cyllow
		#lda_zp_y pom1
		adc #0
		ldy #6
		#eor_zp_y pom1
		bne gy
		ldy #1			;cylhi
		#lda_zp_y pom1
		adc #0
		ldy #5
		#eor_zp_y pom1
		bne gy

vrg		#inc_l_a load_sectors_exp
		lda pom1
		clc
		adc #4
		sta pom1
		bcc linlpx
		inc pom2
linlpx		jmp linlp
		.pend

XIT_rw_com	.proc
		#ldatax buffer_currentp_exp
		lda buffer_filepos2_exp,x
		jsr buffer_tree

		ldy #0
		#lda_zp_y pom1
		jsr head_select_drive
		ldy #3
		#lda_zp_y pom1
		sta low_reg
		sta sector_g
		dey
		#lda_zp_y pom1
		sta low_reg
		sta cyllow_g
		dey
		#lda_zp_y pom1
		sta low_reg
		sta cylhi_g
		jsr LWOP_reporterror_74_xit;-=> error
		ldy #1
		rts
		.pend

XIT_read_in_buffer .proc
		jsr XIT_rw_com		;-=> error
		bit head_g
		lda low_reg
		and #15
		bit sector_g
		ora low_reg
		bit cyllow_g
		ora low_reg
		bit cylhi_g
		ora low_reg
		bne cer			;hole in file?

		.if SUPERCPU=1
		sty pom3
		.fi
		sta pom1
		tay
		#ldatax buffer_currentp_exp
		lda buffer_start,x
		sta pom2
ns		tya
nw		#sta_zp_y pom1
		iny
		bne nw
		inc pom2
                lda pom2
                cmp buffer_address_max_hi_exp,x
                bcc ns
		rts

cer		jsr OP_read
		jsr LWOP_reporterror_read_xit;-=> error

		#ldatax buffer_currentp_exp
		lda buffer_start,x
		tax
		jsr read_buffer_cfs
		jsr ata_pi_nomore_data
		jmp LWOP_reporterror_read_xit;-=> error
		.pend

XIT_write_out_buffer .proc
		jsr XIT_rw_com		;-=> error
		jsr OP_write
		jsr LWOP_reporterror_write_xit;-=> error

		#ldatax buffer_currentp_exp
		lda buffer_start,x
		tax
		jsr write_buffer_cfs
		jsr ata_pi_nomore_data
		bcs vw
		jsr wait_busy
vw		jmp LWOP_reporterror_write_xit;-=> error
		.pend

;----------------------------------
; Calculate the bam sector position for the sector in next_sector
; result in next_sector
;
find_free_start	.proc
szam_thiss	= szam+5

		sta find_free_buf_exp	;this buffer is used for bam
		#lda0
		sta find_free_dirty_exp	;it's not dirty
		sta find_free_buf_exp+1	;mark no bam in buffer

		jsr chs2lba

		ldx #3
ll		lda szam,x
		sta szam_thiss,x
		dex
		bpl ll

		#ldatax partition_currentp_exp
		lda partition_root_dir_exp,x
		asl
		php			;save which is the active bitmap
		#lda0
		tay
		rol
		sta find_free_bam_exp

lq2		lda partition_start_exp,x
		sta next_sector,y
		inx
		iny
		cpy #4
		bne lq2
		jsr chs2lba

		lda szam_thiss+0
		cmp szam+0
		ldx #0
		stx szam_thiss+0
		lda szam_thiss+1
		sbc szam+1
		and #$f0
		sta szam_thiss+1
		lda szam_thiss+2
		sbc szam+2
		sta szam_thiss+2
		lda szam_thiss+3
		sbc szam+3
		sta szam_thiss+3

		ldy #3
		plp
li		lda szam,x
		adc szam_thiss,x
		sta szam,x
		inx
		dey
		bpl li
		jsr lba2chs

		#cnext next_sector , find_free_now_exp
		rts
		.pend
;-----------------------------------
; Find and allocate a sector(s)
;
find_free_sector .proc
		lda #1
		sta load_limit_exp

find_free_sectors .proc
szam_offs	= szam+5

re		lda find_free_buf_exp+1
		bne nzs			;no bam in buffer?

		#cnext find_free_now_exp , next_sector
		gmi getone

nzs		sta pom2
		ldy #0
		sty pom1
		.if SUPERCPU=1
		iny
		sty pom3
		.fi

		#ldy_l_a find_free_buf_exp
		#jmps id

id2		rol
		sta find_free_buf_exp+1
id		#lda_zp_y pom1
		bne free
		iny
		bne id
		inc pom2
		lda pom2
		lsr
		bcs id2

		jsr XIT_find_free_flush	;-=> error

		jsr chs2lba

		lda szam+1		;next block
		clc
		adc #16
		sta szam+1
		bcc ue
		inc szam+2
		bne ue
		inc szam+3
ue
		jsr lba2chs

		#cnext next_sector , find_free_now_exp

getone		lda find_free_buf_exp+1
		and #%11111110
		bne csc
		lda find_free_buf_exp
csc		sta find_free_buf_exp+1
		jsr XIT_read_a_sector	;-=> error
		#lda0
		sta find_free_buf_exp
		geq re			;!!!!!!!!! end of disk!

free		sta find_free_dirty_exp
		ldx #255
xxu		inx
		asl
		bcc xxu
		pha
		stx szam_offs
		lda pom2
		and #1
		sta szam_offs+1
		tya
		asl
		rol szam_offs+1
		asl
		rol szam_offs+1
		asl			;x8
		rol szam_offs+1
		ora szam_offs
		sec
		sbc find_free_bam_exp	;correction if bam!=0
		sta szam_offs
		bcs cvs
		dec szam_offs+1
cvs
		pla

mdf		#inc_l save_p_exp
		bmi gy

		#dec_l load_limit_exp
		beq gy

		inx
ce		asl
		bcs mdf
		cpx #8
		bne gy

		#sta_zp_y pom1

		iny
		bne mbc
		inc pom2
		lda pom2
		lsr
		bcc nvat
		rol
		sta find_free_buf_exp+1

mbc		#lda_zp_y pom1
		ldx #0
		geq ce

gy		pha
		txa
		clc
		adc #$f8
		tax
		pla
		bcs cbr
vm		rol
		inx
		bne vm

cbr		#sta_zp_y pom1
nvat
		#sty_l_a find_free_buf_exp

		#cnext find_free_now_exp , next_sector

		jsr chs2lba

		lda szam
		clc
		adc szam_offs
		sta szam
		lda szam+1
		adc szam_offs+1
		sta szam+1
		bcc vd
		inc szam+2
		bne vd
		inc szam+3
vd		jmp lba2chs
		.pend
;-----------------------------------
; Flush bam
;
XIT_find_free_flush .proc
		lda find_free_dirty_exp
		beq non

		#cnext find_free_now_exp , next_sector

		lda find_free_buf_exp+1
		and #%11111110
		jsr XIT_write_a_sector

		#lda0
		sta find_free_dirty_exp
non		rts
		.pend

;		.if *>$bbf7
;		.error "bug!"
;		.fi
;		*=$bbf7
;		.byte 0			;bbf7
;-----------------------------------
; Find free sectors and fill in datatree
; A: tree buffer
; Y: 0 truncate, 1 truncate but one more sector, 2 round up
;
find_free_sectors_tree .proc
p1		= current_path
p2		= current_path+1
p3		= current_path+2

		tax
		lda save_p_exp
		asl
		asl
		sta p1
		txa
		adc #0
		sta p2
		.if SUPERCPU=1
		lda #1
		sta p3
		.fi

		lda write_size_exp+1
		adc write_p1_exp
		jsr apply_limit

		lda save_p_exp
		pha
		jsr find_free_sectors
		pla
		clc
		sbc save_p_exp
		eor #255
		sta write_sectors_exp
		sta load_limit_exp

		ldy #3
ncs		lda next_sector,y
		#sta_zp_y p1
		dey
		bpl ncs

		#ldatax current_drive_exp

lp		#dec_l_a load_limit_exp
		bne cc
		rts

cc		ldy #3
		#lda_zp_y p1
		bit next_sector
		bvc chs

		clc
		adc #1
		ldy #7
		#sta_zp_y p1
		ldy #2
		#lda_zp_y p1
		adc #0
		ldy #6
		#sta_zp_y p1
		ldy #1
		#lda_zp_y p1
		adc #0
		ldy #5
		#sta_zp_y p1
		ldy #0
		#lda_zp_y p1
		adc #0
		ldy #4
		#sta_zp_y p1
		gne cxm

chs		cmp drive_maxsector_exp,x
		bcc *+4
		lda #255
		adc #1
		ldy #7
		#sta_zp_y p1
		ldy #0
		#lda_zp_y p1
		and #15
		bcc md
		cmp drive_maxhead_exp,x
		bcc *+4
		lda #254
		adc #1
md		ldy #4
		#sta_zp_y p1
		ldy #2
		#lda_zp_y p1
		adc #0
		ldy #6
		#sta_zp_y p1
		ldy #1
		#lda_zp_y p1
		adc #0
		ldy #5
		#sta_zp_y p1

cxm		lda p1
		clc
		adc #4
		sta p1
		bcc lp
		inc p2
		gcs lp
		.pend
;-----------------------------------
; Convert next_sector -> szam
;
chs2lba		.proc
		bit next_sector
		bvc konv
		lda next_sector+0
		and #%01001111
		sta szam+3
		lda next_sector+1
		sta szam+2
		lda next_sector+2
		sta szam+1
		lda next_sector+3
		sta szam+0
		rts

konv		lda next_sector+2
		sta szam
		lda next_sector+1
		sta szam+1
		#stz_a szam+2
		#stz_aa szam+3

		#ldatax current_drive_exp
		lda drive_maxsector_exp,x
		pha
		lda drive_maxhead_exp,x
		tax
		inx
		jsr mul			;cyl*maxhead

		lda next_sector+0
		and #15
		jsr sum			;cyl*maxhead+head

		#plx_a
		jsr mul			;(cyl*maxhead+head)*maxsector
		jsr mul_p4bit

		lda next_sector+3
                #sbc1c
		jmp sum			;(cyl*maxhead+head)*maxsector+sector-1
		.pend
;-----------------------------------
; Convert szam -> next_sector
;
lba2chs		.proc
		bit szam+3
		bvc konv
		lda szam+3
		sta next_sector+0
		lda szam+2
		sta next_sector+1
		lda szam+1
		sta next_sector+2
		lda szam+0
		sta next_sector+3
		rts

konv		#ldatax current_drive_exp
		lda drive_maxhead_exp,x
		pha
		lda drive_maxsector_exp,x
		tax
		jsr div
                #adc1c
		sta next_sector+3

		#plx_a
		inx
		jsr div
		sta next_sector
		lda szam
		sta next_sector+2
		lda szam+1
		sta next_sector+1
		rts
		.pend
;-----------------------------------
; Read directory sector/allocate new
; A=bitmap buffer
;
XIT_prealloc_dir_sector .proc
		pha

		#ldatax partition_currentp_exp
		lda partition_start_exp,x
		and #PATTR_WRITEABLE
		sec
		beq LWOP_reporterror_26_xit;write protected

		ldx #0
nw		lda path_exp,x
		jsr is_special_s
		beq mw
		inx
		bne nw
mw		cmp #1			;0 ?
		jsr LWOP_reporterror_33_xit

		ldx #0
nw2		lda path_searchtype_exp,x
		cmp #32			;no space in type!
		beq mw2
                cmp #"<"		;no < in type!
                beq mw2
		jsr is_special_s
		beq mw2
		inx
		cpx #3
		bne nw2
		#lda0
mw2		cmp #1			;0 ?
		jsr LWOP_reporterror_33_xit

		#cnext resolv_this_sector_exp , next_sector;get last read dir sector

		pla
		jsr find_free_start	;start allocation from last read dir sector

		ldx #3			;check if there's no empty entry in dir?
swc2		lda resolv_empty_entry_exp,x
		bne havedir
		lda resolv_this_sector_exp,x
		sta next_sector,x
		dex
		bpl swc2

		lda #ram_exp_buffer0b	;read in the last sector of directory
		jsr XIT_read_a_sector	;-=> error

		jsr find_free_sector	;allocate next directory sector

		#cnext next_sector , resolv_empty_entry_exp

		ldx #0
		txa			;clear new dir sector
ke		sta ram_exp_buffer0_exp,x
		sta ram_exp_buffer0_exp+$100,x
		inx
		bne ke

		#lda0			;have to write out new empty dir sector first
		jsr XIT_write_a_sector	;-=> error

		lda #ram_exp_buffer0b
		jsr init_pom1_y

		ldy #$14
		ldx #2
mve		#lda_zp_y pom1
		asl next_sector
		rol next_sector+1	;update directory link in old dir sector
		rol next_sector+2
		rol next_sector+3
		bcc *+4
		ora #%00100000
		asl next_sector
		rol next_sector+1
		rol next_sector+2
		rol next_sector+3
		bcc *+5
		clc
		ora #%00010000
		#sta_zp_y pom1
		tya
		adc #$20
		tay
		bcc mve
		inc pom2
		dex
		bne mve

		jsr XIT_find_free_flush	;-=> error we must first flush the bam, then extend the directory!

		#cnext resolv_this_sector_exp , next_sector

		lda #ram_exp_buffer0b	;write out modifyed dir sector
		jmp XIT_write_a_sector	;-=> error
					;now we have a longer directory with free entrys...
;---------------
					;There was already a directory sector

havedir		#cnext resolv_empty_entry_exp , next_sector

		lda #ram_exp_buffer0	;read in directory sector
		jmp XIT_read_a_sector	;-=> error
		.pend
;-----------------------------------
; Fill in directory entry info
;
XIT_finalize_direntry .proc
		pha
		ldy #$18		;attrib offset
wdr		#lda_zp_y pom1
		and #FATTR_CLOSED | FTYPE_MASK
		beq empt

		lda pom1
		clc
		adc #32
		sta pom1
		bcc wdr
		inc pom2
		gcs wdr

empt		tay
		tax
ii		lda path_exp,x		;fill in filename
		beq *+3
		inx
		#sta_zp_y pom1
		iny
		cpy #16
		bne ii

		ldx #4
mc		lda szam-1,x
		#sta_zp_y pom1
		iny
		dex
		bne mc			;fill in filesize

		#lda_zp_y pom1
		and #%00110000		;nexts
		ora created_new_tree_sector_exp
		jmp fv

vwr		inx
		lda created_new_tree_sector_exp,x
fv		#sta_zp_y pom1		;fill in tree sector
		iny
		cpx #3
		bne vwr

		pla			;fileattrib
		#sta_zp_y pom1
		iny

scw		#lda_l_y_x path_searchtype_exp-$19
		#sta_zp_y pom1		;fill in attributes and filetype
		iny
		cpy #$1c
		bne scw

		#lda0
		#sta_zp_y pom1		;fill in date TODO!
		iny
		#sta_zp_y pom1
		iny
		#sta_zp_y pom1
		iny
		#sta_zp_y pom1

		#cnext resolv_empty_entry_exp , next_sector
		rts
		.pend

;-----------------------------------
; Update filelength if position is after fileend
;
update_filelen	.proc
		lda buffer_address_low_exp,x
		clc
		adc #1
		sta pom1
		lda buffer_address_hi_exp,x
		adc #0
		sec
		sbc buffer_start,x
		clc
		adc buffer_filepos2_exp,x
		sta pom2
		lda buffer_filepos3_exp,x
		adc #0
		sta pom3
		lda buffer_filepos4_exp,x
		adc #0
		sta pom4
		lda buffer_filesize1_exp,x
		cmp pom1
		lda buffer_filesize2_exp,x
		sbc pom2
		lda buffer_filesize3_exp,x
		sbc pom3
		lda buffer_filesize4_exp,x
		sbc pom4
		bcs gse
		lda pom1
		sta buffer_filesize1_exp,x
		lda pom2
		sta buffer_filesize2_exp,x
		lda pom3
		sta buffer_filesize3_exp,x
		lda pom4
		sta buffer_filesize4_exp,x
gse		rts
		.pend
;-----------------------------------
; Limit maximal sectors transfered
;
apply_limit	.proc
		ror
		#ldx_l_ current_drive_exp
		cmp setup_linear_hdd0_exp,x
		bcc idi
		beq idi
		lda setup_linear_hdd0_exp,x
		adc #1-1
idi		sta load_limit_exp
		rts
		.pend
