;
; save.asm - SAVE kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Save (A=start address on zeropage, X,Y last not saved byte)
;
; Used registers: A X Y
; Zeropage: $c1,$c2=start address
; Zeropage: $ae,$af=X and Y register before call
; Zeropage: $ac,$ad=the byte after the lastsaved
; Status: C
; C=1:
; A=0 break
; A=5 device not present
; A=8 missing filename
; A=9 illegal device number
; A=24 file data
; Status response:
; ST=$00 ok
; ST=$03 timeout
; ST=$80 device not present
;
; Notes:
; Can only save 65533 bytes+load address (1 tree sector)
;**************************************************
;--------------------------------------
; SAVE
;
;		stx $ae
;		sty $af
;		tax
;		lda $00,x
;		sta $c1
;		lda $01,x
;		sta $c2
;		jmp ($0332)
;--------------------------------------
newsave		.proc
		
                .if DEBUGIO=1
                lda debug_io_exp
                beq kd
                #lda0
                sta debug_io_exp
                #psr $f5ed
                php
                lda $c2
                pha
                #phx_a
                #phy_a
                lda #3
                jmp monitor
kd		lda #1
		sta debug_io_exp
                .fi

		lda ActualFNLength
		beq exit_missing_filename

		lda ActualDev
                beq kernal_save.ill
		#devicehandler hard_save,pclink_save2,exit_illegal_device,kernal_save
		.pend

kernal_save	.proc
		tya
		beq ill			;drive 0 = keyboard
		cmp #31
		bcs ill			;illegal serial device
		cmp #3
		beq ill			;drive 3 = screen
		bcs ok
		#psr $f5f8
		tya
		jmp exit2basic		;Normal kernal rutin
ill		jmp exit_illegal_device

ok		sta save_p_exp		;store drive number
		pha
		#OP_scpu01up
		jsr OP_save_zeropage
		jsr OP_copy_filename_to_path

		lda #$61
		sta ActualSecAddr
		#stz_a Status
		jsr OP_cfg16up
		pla
		tay
		.if SERIAL_HANG_FIX=1
		jsr test_hang
		bcc np
		.fi
		jsr ieclisten		;listen
		lda #$f1
		jsr iecsecond		;second
		lda Status
		bpl oe
np		jsr OP_cfg16down
		jsr OP_restore_zeropage
		#OP_scpu01down
		jmp exit_device_not_present

oe
		ldx #0
cy		#sta_cfg_open
		lda path_exp,x
		#sta_cfg_16
		beq oy
		jsr iecout		;ciout
		inx
		bne cy
oy
		jsr iecunlisten		;unlsn

		tya
		jsr ieclisten		;listen
		lda #$61
		jsr iecsecond		;second
		lda $c1
		sta $ac
		jsr iecout
		lda $c2
		sta $ad
		jsr iecout
		jsr OP_cfg16down

		jsr OP_save_saving

		ldx #9
di		lda read_byte_r3,x
		sta start_zeropage,x
		dex
		bpl di

		ldy #0
loop		lda $ac
		cmp $ae
		lda $ad
		sbc $af
		bcs ende
		jsr start_zeropage
		tax
		jsr OP_cfg16up
		txa
		jsr iecout
		jsr OP_cfg16down
		lda Status
		bne err
		inc $ac
		bne loop
		jsr check_stop
		beq sto
		inc $ad
		gne loop
err		iny
sto		iny
ende
		lda save_p_exp
		pha
		jsr OP_cfg16up
		jsr iecunlisten		;unlisten
		pla
		jsr ieclisten		;f648
		lda #$e1
		jsr iecsecond
		jsr iecunlisten
		jsr OP_cfg16down
in		jsr OP_restore_zeropage
		#OP_scpu01down
		lda Status
		bne errr
		tya
		beq oke
		dey
errr		sec
		bne exit_file_data
		#lda0			;stop pressed
		jmp exit2basic

oke		jsr OP_load_loading_end
		clc
		jmp exit2basic
		.pend
;--------------------------------------
; SAVE to PCLink
;--------------------------------------
pclink_save2    #OP_scpu01up
		jsr OP_save_zeropage
		jsr OP_copy_filename_to_path
		jsr pclink_save
                #jmps kernal_save.in
;--------------------------------------
; SAVE to IDE
;--------------------------------------
hard_save	.proc
		lda #$61
		sta ActualSecAddr
                #OP_scpu01up
		jsr OP_findunusedchannel
                #OP_scpu01down
		jsr newopen_t
		jsr dv
		ldx ActualLogical
		jsr newchkout_t
		jsr dv
		lda $c1
		sta $ac
		jsr newchrout_t
		jsr dv
		lda $c2
		sta $ad
		jsr newchrout_t
		jsr dv
		lda $ae
		sec
		sbc $ac
		tax
		lda $af
		sbc $ad
		tay
		bcc vr
		lda #$ac
		jsr newwrite_t
		jsr dv
		txa
		adc $ac
		sta $ac
		tya
		adc $ad
		sta $ad
vr		jsr newclrchn_t
		lda ActualLogical
		jsr newclose_t
		jsr dv
		jmp exit2basic

dv		bcs dv2
		lda Status
		bne dv2
		rts
dv2		pla
		pla
		sec
		jmp exit_file_data	;error!

		.pend
;-----------------------------------
; Read a byte (for save)
;
read_byte_r3	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		lda ($ac),y		;2
		sta cfg_open		;3
		rts			;1
		.here
		.pend
